@@ -28,7 +28,7 @@
 
 /**
 @file      simd.h
-@brief     RE/flex SIMD intrinsics
+@brief     RE/flex SIMD primitives
 @author    Robert van Engelen - engelen@genivia.com
 @copyright (c) 2016-2024, Robert van Engelen, Genivia Inc. All rights reserved.
 @copyright (c) BSD-3 License - see LICENSE.txt
@@ -60,7 +60,9 @@
 #ifdef _MSC_VER
 # define cpuidex __cpuidex
 #else
-# include <cpuid.h>
+# ifndef __cpuid_count
+#  include <cpuid.h>
+# endif
 # define cpuidex(CPUInfo, id, subid) __cpuid_count(id, subid, CPUInfo[0], CPUInfo[1], CPUInfo[2], CPUInfo[3])
 #endif
 
@@ -138,21 +140,18 @@ inline uint32_t popcountl(uint64_t x)
 #endif
 
 // Partially count newlines in string b up to e, updates b close to e with uncounted part
-extern size_t simd_nlcount_sse2(const char*& b, const char *e);
-extern size_t simd_nlcount_avx2(const char*& b, const char *e);
-extern size_t simd_nlcount_avx512bw(const char*& b, const char *e);
+extern size_t simd_nlcount_avx2(const char *&b, const char *e);
+extern size_t simd_nlcount_avx512bw(const char *&b, const char *e);
 
 } // namespace reflex
 
-#elif defined(HAVE_NEON)
+#endif
 
 namespace reflex {
 
-// Partially count newlines in string b up to e, updates b close to e with uncounted part
-extern size_t simd_nlcount_neon(const char*& b, const char *e);
+// Count newlines in string s up to t
+extern size_t nlcount(const char *s, const char *t);
 
 } // namespace reflex
 
 #endif
-
-#endif
